@@ -16,12 +16,12 @@ from genepy3d.io.base import CatmaidApiTokenAuth
 
 
 class Tree:
-    """Tree in 3D.
+    """Tree in 3D. Uses the anytree package for may underlying representation and functions.
     
     Attributes:
-        id (int): tree ID.
-        name (str): tree name.
-        nodes (anytree): structure of tree nodes.    
+        nodes (anytree): structure of tree nodes. 
+        id (int): tree ID (optional).
+        name (str): tree name (optional).  
     
     """
     
@@ -38,7 +38,7 @@ class Tree:
             nodes_tbl (pandas dataframe): tree nodes table.
             connectors_tbl (pandas dataframe): connectors table.
         
-        """        
+        """   
         
         nodes = {}
         
@@ -76,7 +76,7 @@ class Tree:
     
     @classmethod
     def from_table(cls,nodes_tbl,connectors_tbl=None,tree_id=0,tree_name='genepy3d'):
-        """Build tree from dataframes.
+        """Build tree from Pandas dataframes.
         
         Args:
             nodes_tbl (pandas dataframe): tree nodes table.
@@ -131,6 +131,62 @@ class Tree:
         
         return cls(nodes,tree_id,tree_name)
                        
+    @classmethod
+    def from_eswc(cls,filepath):
+        """Build tree from ESWC file. The ESWC is used e.g. Vaa3D.
+        
+        Args:
+            filepath (str): path to eswc file.
+        
+        """
+        
+        data = []
+        
+        # extract info from file
+        f = open(filepath,'r')
+        for line in f:
+            if line[0]!='#':
+                tmp = []
+                
+                elemens = re.split(r'\r|\n| |\s', line)
+                for ile in elemens:
+                    if ile!='':
+                        tmp.append(float(ile))
+                
+                data.append(tmp)
+                
+        f.close()
+        
+        # build dataframe and cast columns types
+        if len(data)!=0:
+            
+            tree_name = os.path.basename(filepath).split(".eswc")[0]
+            tree_id = 0
+            
+            if len(data[0])==12:
+                dfneu = pd.DataFrame(data,columns=['treenode_id', 'structure_id', 'x', 'y', 'z', 'r', 'parent_treenode_id','segment_id','level','mode','timestamp','featureval'])            
+            elif len(data[0])==11:
+                dfneu = pd.DataFrame(data,columns=['treenode_id', 'structure_id', 'x', 'y', 'z', 'r', 'parent_treenode_id','segment_id','level','mode','timestamp'])
+            else:
+                raise Exception('The number of columns must be 11 or 12.')
+            
+            dfneu = dfneu[['treenode_id', 'structure_id', 'x', 'y', 'z', 'r', 'parent_treenode_id']]
+            dfneu.dropna(inplace=True)
+            
+            # cast type
+            dfneu['treenode_id'] = dfneu['treenode_id'].astype('int')
+            dfneu['structure_id'] = dfneu['structure_id'].astype('int')
+            dfneu['parent_treenode_id'] = dfneu['parent_treenode_id'].astype('int')
+            
+            # reset index
+            dfneu.drop_duplicates(subset=["treenode_id"],inplace=True) # make sure unique index
+            dfneu.set_index('treenode_id',inplace=True) # reset dataframe index
+            # build tree node structure
+            nodes = cls.build_treenodes(dfneu,None)
+            return cls(nodes,tree_id,tree_name)
+        else:
+            raise ValueError("Errors when reading file.")
+    
     @classmethod
     def from_swc(cls,filepath):
         """Build tree from SWC file.
@@ -394,7 +450,15 @@ class Tree:
         return df    
     
     def compute_length(self,nodeid=None,rootid=None):
-        """Return lengths for given nodes.
+        """Return length of the tree of part of it. If nodeid is `none`, compute length of the whole tree. Else compute the length of the given set of nodes, assuming they are contiguous and ordered.
+        
+        Args:
+            nodeid (list[int]): list of node IDs or None. (default None)
+        
+        Returns:
+            int.
+
+        
         """
         
         from genepy3d.obj.curves import Curve
@@ -1026,7 +1090,7 @@ class Tree:
         return Tree(newnodes)
     
     def compute_orientation(self,decomposed_method="leaf",sigma=0,rootid=None):
-        """Return orientation vector and angles with x, y and z.
+        """Return orientation vector and angles with x, y and z axes.
         
         Args:
             decomposed_method (str): "leaf", "spine" or "branching", method for cutting neuronal trace
@@ -1094,10 +1158,198 @@ class Tree:
         
         df.set_index("nodeid",inplace=True)            
         return df
+    
+    def compute_local_3d_scale_sigma(self,sig_lst,dim_param=None,decomposed_method="leaf",rootid=None):
+        """Compute 3d local scale of neuron from a list of sigma of Gaussian.
+        
+        Args:
+            sig_lst (list): list of sigma values of Gaussian.
+            dim_param (dic): parameters for dimension decomposition
+            decomposed_method (str): "leaf", "spine" or "branching", method for cutting neuronal trace
+            rootid (int): ID of root
+            
+        Returns:
+            Pandas dataframe containing local 3D scale, 1D, 2D, 3D flags, curvature and torsion
+        
+        """
+        
+        from genepy3d.obj.curves import Curve
+        
+        if rootid is None:
+            _rootid = self.get_root()[0]
+        else:
+            _rootid = rootid
+        
+        nodeid_lst, ls_lst, plane_line_lst, line_lst, threed_lst = [], [], [], [], []
+        curvature_lst, torsion_lst = [], []
+        segkey_lst = []
+    
+        # decompose tree into segments
+        if decomposed_method=="spine":
+            segments = self.decompose_spines(_rootid)
+        elif decomposed_method=="leaf":
+            segments = self.decompose_leaves(_rootid)
+        else:
+            segments = self.decompose_segments(_rootid)
+        
+        for key,seg in segments.items():
+
+            # create curve
+            coors = self.get_coordinates(seg).values
+            crv = Curve(coors)
+            
+            # intrinsic dimension decomposition
+            try:
+                if dim_param is not None:
+                    dim_param["return_dim_results"] = True
+                    ls_res, intrinsic_res = crv.compute_local_3d_scale_sigma(sig_lst,**dim_param)
+                else: # use default params
+                    ls_res, intrinsic_res = crv.compute_local_3d_scale_sigma(sig_lst,return_dim_results=True)
+            except:
+                raise Exception(seg)
+    
+            # save segment keys
+            segkey_lst += [key for _ in range(len(seg))]
+            
+            # curvature and torsion
+            curvature_lst += crv.compute_curvature().tolist()
+            torsion_lst += crv.compute_torsion().tolist()
+    
+            # save node IDs
+            nodeid_lst = nodeid_lst + list(seg)
+            
+            # local 3D scale
+            ls_lst = ls_lst + list(ls_res)
+    
+    #         if(len(np.argwhere(np.array(ls_res)>=70).flatten())==len(ls_res)):
+    #             print(seg)
+    
+            # 1D, 2D, 3D flags
+            pl_flag = np.zeros((len(seg),len(sig_lst)))
+            l_flag = np.zeros((len(seg),len(sig_lst)))
+            for i in range(len(sig_lst)):
+                res = intrinsic_res[i]
+                for plids in res["planeline_pred"]:
+                    pl_flag[plids[0]:plids[1]+1,i]=1.
+                for lids in res["line_pred"]:
+                    l_flag[lids[0]:lids[1]+1,i]=1.
+            threed_flag = (pl_flag + np.ones(pl_flag.shape))%2 # xor bit (3D is the opposite of planeline)
+    
+            plane_line_lst = plane_line_lst + pl_flag.tolist()
+            line_lst = line_lst + l_flag.tolist()
+            threed_lst = threed_lst + threed_flag.tolist()
+        
+        # making dataframe
+        df = pd.DataFrame({"nodeid":nodeid_lst,
+                           "seg_key":segkey_lst, 
+                           "local_scale":ls_lst,
+                           "plane_line_flag":plane_line_lst,
+                           "line_flag":line_lst,
+                           "threed_flag":threed_lst,
+                           "curvature":curvature_lst,"torsion":torsion_lst})
+        
+    #     df.drop_duplicates("nodeid",keep=False) # drop branching nodes
+        df.set_index("nodeid",inplace=True)            
+        return df 
+    
+    def compute_local_3d_scale_radius(self,r_lst,dim_param=None,decomposed_method="leaf",rootid=None):
+        """Compute 3d local scale of neuron from a list of radius of curvatures.
+        
+        Args:
+            r_lst (list): list of scales (radius of curvature)
+            dim_param (dic): parameters for dimension decomposition
+            decomposed_method (str): "leaf", "spine" or "branching", method for cutting neuronal trace
+            rootid (int): ID of root
+            
+        Returns:
+            Pandas dataframe containing local 3D scale, 1D, 2D, 3D flags, curvature and torsion
+        
+        """
+        
+        from genepy3d.obj.curves import Curve
+        
+        if rootid is None:
+            _rootid = self.get_root()[0]
+        else:
+            _rootid = rootid
+        
+        nodeid_lst, ls_lst, plane_line_lst, line_lst, threed_lst = [], [], [], [], []
+        curvature_lst, torsion_lst = [], []
+        segkey_lst = []
+    
+        # decompose tree into segments
+        if decomposed_method=="spine":
+            segments = self.decompose_spines(_rootid)
+        elif decomposed_method=="leaf":
+            segments = self.decompose_leaves(_rootid)
+        else:
+            segments = self.decompose_segments(_rootid)
+        
+        for key,seg in segments.items():
+
+            # create curve
+            coors = self.get_coordinates(seg).values
+            crv = Curve(coors)
+            
+            # intrinsic dimension decomposition
+            try:
+                if dim_param is not None:
+                    dim_param["return_dim_results"] = True
+                    ls_res, intrinsic_res = crv.compute_local_3d_scale_radius(r_lst,**dim_param)
+                else: # use default params
+                    ls_res, intrinsic_res = crv.compute_local_3d_scale_radius(r_lst,return_dim_results=True)
+            except:
+                raise Exception(seg)
+    
+            # save segment keys
+            segkey_lst += [key for _ in range(len(seg))]
+            
+            # curvature and torsion
+            curvature_lst += crv.compute_curvature().tolist()
+            torsion_lst += crv.compute_torsion().tolist()
+    
+            # save node IDs
+            nodeid_lst = nodeid_lst + list(seg)
+            
+            # local 3D scale
+            ls_lst = ls_lst + list(ls_res)
+    
+    #         if(len(np.argwhere(np.array(ls_res)>=70).flatten())==len(ls_res)):
+    #             print(seg)
+    
+            # 1D, 2D, 3D flags
+            pl_flag = np.zeros((len(seg),len(r_lst)))
+            l_flag = np.zeros((len(seg),len(r_lst)))
+            for i in range(len(r_lst)):
+                res = intrinsic_res[i]
+                for plids in res["planeline_pred"]:
+                    pl_flag[plids[0]:plids[1]+1,i]=1.
+                for lids in res["line_pred"]:
+                    l_flag[lids[0]:lids[1]+1,i]=1.
+            threed_flag = (pl_flag + np.ones(pl_flag.shape))%2 # xor bit (3D is the opposite of planeline)
+    
+            plane_line_lst = plane_line_lst + pl_flag.tolist()
+            line_lst = line_lst + l_flag.tolist()
+            threed_lst = threed_lst + threed_flag.tolist()
+        
+        # making dataframe
+        df = pd.DataFrame({"nodeid":nodeid_lst,
+                           "seg_key":segkey_lst, 
+                           "local_scale":ls_lst,
+                           "plane_line_flag":plane_line_lst,
+                           "line_flag":line_lst,
+                           "threed_flag":threed_lst,
+                           "curvature":curvature_lst,"torsion":torsion_lst})
+        
+    #     df.drop_duplicates("nodeid",keep=False) # drop branching nodes
+        df.set_index("nodeid",inplace=True)            
+        return df 
 
     def compute_local_3d_scale(self,r_lst,dim_param=None,decomposed_method="leaf",rootid=None):
         """Compute 3d local scale of neuron.
         
+        This function is DEPRECATED. Use compute_local_3d_scale_radius().
+        
         Args:
             r_lst (list): list of scales (radius of curvature)
             dim_param (dic): parameters for dimension decomposition
@@ -1194,7 +1446,7 @@ class Tree:
         return df
     
     def summary(self):
-        """Return a brief  summary of tree.
+        """Return a brief summary of tree.
         
         Returns:
             pandas serie.
@@ -1221,7 +1473,7 @@ class Tree:
         return pd.Series(data,index=index)
     
     def to_curve(self,nodeid=None):
-        """Convert a segment on tree given by nodeid list to Curve.
+        """Convert a segment of tree given by nodeid list to Curve.
         """
         from genepy3d.obj.curves import Curve
         return Curve(self.get_coordinates(nodeid).values)
